@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2025 Antonio Diaz Diaz.
+   Copyright (C) 2013-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -282,7 +282,7 @@ bool compare_file_type( std::string & estr, std::string & ostr,
   const char * const filename = extended.path().c_str();
   const Typeflag typeflag = (Typeflag)header[typeflag_o];
   struct stat st;
-  bool diff = false, size_differs = false, type_differs = true;
+  bool dif = false, size_differs = false, type_differs = true;
   if( hstat( filename, &st, cl_opts.dereference ) != 0 )
     format_file_error( estr, filename, "warning: can't stat", errno );
   else if( ( typeflag == tf_regular || typeflag == tf_hiperf ) &&
@@ -306,14 +306,14 @@ bool compare_file_type( std::string & estr, std::string & ostr,
       const mode_t mode = parse_octal( header + mode_o, mode_l );  // 12 bits
       if( mode != ( st.st_mode & ( S_ISUID | S_ISGID | S_ISVTX |
                                    S_IRWXU | S_IRWXG | S_IRWXO ) ) )
-        { format_file_diff( ostr, filename, "Mode differs" ); diff = true; }
+        { format_file_diff( ostr, filename, "Mode differs" ); dif = true; }
       }
     if( !cl_opts.ignore_ids && !cl_opts.ignore_metadata )
       {
       if( extended.get_uid() != (long long)st.st_uid )
-        { format_file_diff( ostr, filename, "Uid differs" ); diff = true; }
+        { format_file_diff( ostr, filename, "Uid differs" ); dif = true; }
       if( extended.get_gid() != (long long)st.st_gid )
-        { format_file_diff( ostr, filename, "Gid differs" ); diff = true; }
+        { format_file_diff( ostr, filename, "Gid differs" ); dif = true; }
       }
     if( typeflag != tf_symlink )
       {
@@ -321,9 +321,9 @@ bool compare_file_type( std::string & estr, std::string & ostr,
           extended.mtime().sec() != (long long)st.st_mtime )
         {
         if( (time_t)extended.mtime().sec() == st.st_mtime )
-          { if( !cl_opts.ignore_overflow ) { diff = true;
+          { if( !cl_opts.ignore_overflow ) { dif = true;
               format_file_diff( ostr, filename, "Mod time overflow" ); } }
-        else { diff = true;
+        else { dif = true;
                format_file_diff( ostr, filename, "Mod time differs" ); }
         }
       if( ( typeflag == tf_regular || typeflag == tf_hiperf ) &&
@@ -334,7 +334,7 @@ bool compare_file_type( std::string & estr, std::string & ostr,
             (unsigned)major( st.st_rdev ) ||
             parse_octal( header + devminor_o, devminor_l ) !=
             (unsigned)minor( st.st_rdev ) ) )
-        { format_file_diff( ostr, filename, "Device number differs" ); diff = true; }
+        { format_file_diff( ostr, filename, "Device number differs" ); dif = true; }
       }
     else
       {
@@ -348,10 +348,10 @@ bool compare_file_type( std::string & estr, std::string & ostr,
         if( extended.linkpath() != buf ) e = true;
         }
       delete[] buf;
-      if( e ) { format_file_diff( ostr, filename, "Symlink differs" ); diff = true; }
+      if( e ) { format_file_diff( ostr, filename, "Symlink differs" ); dif = true; }
       }
     }
-  if( diff || size_differs || type_differs ) set_error_status( 1 );
+  if( dif || size_differs || type_differs ) set_error_status( 1 );
   return !( size_differs || type_differs );
   }
 
@@ -367,34 +367,34 @@ bool compare_file_contents( std::string & estr, std::string & ostr,
   uint8_t buf1[bufsize];
   uint8_t buf2[bufsize];
   int retval = 0;
-  bool diff = false;
+  bool dif = false;
   estr.clear(); ostr.clear();
   while( rest > 0 )
     {
     const int rsize1 = ( rest >= bufsize ) ? bufsize : rest + padding;
     const int rsize2 = ( rest >= bufsize ) ? bufsize : rest;
-    if( ( retval = ar.read( buf1, rsize1 ) ) != 0 ) { diff = true; break; }
-    if( !diff )
+    if( ( retval = ar.read( buf1, rsize1 ) ) != 0 ) { dif = true; break; }
+    if( !dif )
       {
       const int rd = readblock( infd2, buf2, rsize2 );
       if( rd != rsize2 )
         {
         if( errno ) format_file_error( estr, filename, rd_err_msg, errno );
         else format_file_diff( ostr, filename, "EOF found in file" );
-        diff = true;
+        dif = true;
         }
       else
         {
         int i = 0; while( i < rsize2 && buf1[i] == buf2[i] ) ++i;
         if( i < rsize2 )
-          { format_file_diff( ostr, filename, "Contents differ" ); diff = true; }
+          { format_file_diff( ostr, filename, "Contents differ" ); dif = true; }
         }
       }
     if( rest < bufsize ) break;
     rest -= rsize1;
     }
   close( infd2 );
-  if( diff ) set_error_status( 1 );
+  if( dif ) set_error_status( 1 );
   return retval;
   }
 
@@ -406,7 +406,7 @@ int decode( const Cl_options & cl_opts )
   const Archive_descriptor ad( cl_opts.archive_name );
   if( ad.infd < 0 ) return 1;
   if( ad.name.size() && ad.indexed && ad.lzip_index.multi_empty() )
-    { show_file_error( ad.namep, empty_msg ); close( ad.infd ); return 2; }
+    { show_file_error( ad.namep, empty_member_msg ); close( ad.infd ); return 2; }
 
   const Arg_parser & parser = cl_opts.parser;
   const bool c_present = cl_opts.option_C_present &&
